(()=>{
    "use strict";
    return {
        dependencies: ["packages://inspector/share/meta-header.js", "packages://inspector/share/texture-compress-section.js"],
        style: `
              .preview {
                    margin: 20px;
              }

              .title {
                    font-weight: bold;
                    text-overflow: ellipsis;
                    overflow: hidden;
                    display: flex;
                    flex: none;
                    flex-direction: row;
                    align-items: center;
                    padding-bottom: 2px;
                    margin: 5px 10px;
                    margin-bottom: 10px;
                    border-bottom: 1px solid #666;
                    height: 24px;
                    overflow: hidden;
              }

              .image-wrapper {
                    width: 100%;
                    padding: 10px;
                    box-sizing: border-box;
              }

              .preview-img {
                    width: 100%;
                    margin-bottom: 20px;
                    border: 2px solid #666;
              }

              .image-info {
                    text-align: center;
                    padding: 2px 10px;
                    font-size: 10px;
                    font-weight: bold;
                    color: rgb(255, 255, 255);
              }
            `,
        template: `
              <cc-meta-header
                :target="target"
              ></cc-meta-header>

              <div class="props flex-1">
                <div>
                  <ui-prop name="${Editor.T("AUTOATLAS.max_width")}" type="number"
                    v-value="target.maxWidth"
                    v-values="target.multiValues.maxWidth"
                    :multi-values="multi"
                  ></ui-prop>
                  <ui-prop name="${Editor.T("AUTOATLAS.max_height")}" type="number"
                    v-value="target.maxHeight"
                    v-values="target.multiValues.maxHeight"
                    :multi-values="multi"
                  ></ui-prop>
                  <ui-prop name="${Editor.T("AUTOATLAS.padding")}" type="number"
                    v-value="target.padding"
                    v-values="target.multiValues.padding"
                    :multi-values="multi"
                  ></ui-prop>
                  <ui-prop name="${Editor.T("AUTOATLAS.allow_rotation")}" type="boolean"
                    v-value="target.allowRotation"
                    v-values="target.multiValues.allowRotation"
                    :multi-values="multi"
                  ></ui-prop>
                  <ui-prop name="${Editor.T("AUTOATLAS.force_squared")}" type="boolean"
                    v-value="target.forceSquared"
                    v-values="target.multiValues.forceSquared"
                    :multi-values="multi"
                  ></ui-prop>
                  <ui-prop name="${Editor.T("AUTOATLAS.powerOfTwo")}" type="boolean"
                    v-value="target.powerOfTwo"
                    v-values="target.multiValues.powerOfTwo"
                    :multi-values="multi"
                  ></ui-prop>
                  <ui-prop name="${Editor.T("AUTOATLAS.heuristices")}" type="enum"
                    v-value="target.algorithm"
                    v-values="target.multiValues.algorithm"
                    :multi-values="multi"
                  >
                    <option value="MaxRects">MaxRects</option>
                    \x3c!-- TODO: 这个算法还有问题，等待修复后再集成
                      <option value="ipacker">ipacker</option>
                    --\x3e
                  </ui-prop>
                  <ui-prop name="${Editor.T("AUTOATLAS.padding_bleed")}" tooltip="${Editor.T("AUTOATLAS.padding_bleed_info")}" type="boolean"
                    v-value="target.paddingBleed"
                    v-values="target.multiValues.paddingBleed"
                    :multi-values="multi"
                  ></ui-prop>
                  <ui-prop name="${Editor.T("AUTOATLAS.filter_unused")}" tooltip="${Editor.T("AUTOATLAS.filter_unused_warn")}">
                    <ui-checkbox class="item"
                      v-value="target.filterUnused"
                      v-values="target.multiValues.filterUnused"
                      :multi-values="multi"
                    >
                      <i class="fa fa-exclamation-triangle fa-1" aria-hidden="true" style="color:yellow" title="${Editor.T("AUTOATLAS.filter_unused_warn")}"></i>
                    </ui-checkbox>
                  </ui-prop>

                  <ui-section>
                    <div class="header">Texture</div>

                    <ui-prop name="Premultiply Alpha" type="boolean"
                      v-value="target.premultiplyAlpha"
                      v-values="target.multiValues.premultiplyAlpha"
                      :multi-values="multi"
                    ></ui-prop>
                    <ui-prop name="Filter Mode" type="enum"
                      v-value="target.filterMode"
                      v-values="target.multiValues.filterMode"
                      :multi-values="multi"
                    >
                      <option value="point">Point</option>
                      <option value="bilinear">Bilinear</option>
                      <option value="trilinear">Trilinear</option>
                    </ui-prop>
                    <ui-prop name="Packable" type="boolean"
                      v-value="target.packable"
                      v-values="target.multiValues.packable"
                      :multi-values="multi"
                    ></ui-prop>

                    <texture-compress v-ref:preview v-if="!multi"
                      :target.sync="target"
                    >
                    </texture-compress>
                  </ui-section>

                </div>

                <div style="display:flex">
                  <ui-button class="preview flex-1" @confirm="_onPreviewClick"
                    v-if="!multi"
                  >
                    Preview
                  </ui-button>
                </div>

                <div v-if="!multi">
                  <div class="title">Packed Textures</div>

                  <div class="image-wrapper">
                    <div v-for="texture in packedTextures">
                      <div class="image-info">
                        <span class="info-name">{{texture.name}}</span>
                        <span class="info-result">{{texture.result}}</span>
                      </div>
                      <img class="preview-img"
                        :src="texture.path"
                      ></img>
                    </div>
                  </div>

                  <div class="title">Unpacked Textures</div>

                  <div class="image-wrapper">
                    <div v-for="texture in unpackedTextures">
                      <div class="image-info">
                        <span class="info-name">{{texture.name}}</span>
                        <span class="info-result">{{texture.result}}</span>
                      </div>
                      <img class="preview-img"
                        :src="texture.path"
                      ></img>
                    </div>
                  </div>
                </div>

                <ui-loader v-if="generating">Generating...</ui-loader>
              </div>
            `,
        props: {
            packedTextures: {
                type: Array,
                default: function() {
                    return []
                }
            },
            unpackedTextures: {
                type: Array,
                default: function() {
                    return []
                }
            },
            generating: !1
        },
        ready() {
            this._getInfo()
        },
        watch: {
            target: "_getInfo"
        },
        methods: {
            _onPreviewClick() {
                this.generating = !0,
                Editor.Ipc.sendToPanel("scene", "scene:generate-texture-packer-preview-files", this.target.uuid, e=>{
                    if (this.generating = !1,
                    e)
                        return this.packedTextures = [],
                        this.unpackedTextures = [],
                        void 0;
                    this._getInfo()
                }
                , -1)
            },
            _getInfo() {
                Editor.Ipc.sendToPanel("scene", "scene:query-texture-packer-preview-files", this.target.uuid, (e,n)=>{
                    n ? (n.packedTextures.forEach(e=>{
                        e.path += "?time=" + Date.now()
                    }
                    ),
                    n.unpackedTextures.forEach(e=>{
                        e.path += "?time=" + Date.now()
                    }
                    ),
                    this.packedTextures = n.packedTextures,
                    this.unpackedTextures = n.unpackedTextures) : (this.packedTextures = [],
                    this.unpackedTextures = [])
                }
                )
            }
        }
    }
}
)();
