"use strict";
Vue.component("cc-camera", {
    template: `

    <style>
      #checkboxes div ui-checkbox {
            margin-top: 1px;
      }
      #checkboxes div:first-child ui-checkbox {
            margin-top: 0;
      }
    </style>

    <ui-prop
      v-prop="target.backgroundColor"
      :multi-values="multi"
    ></ui-prop>

    <ui-prop
      v-prop="target.depth"
      :multi-values="multi"
    ></ui-prop>

    <ui-prop
      name="cullingMask"
      :multi-values="multi"
      auto-height
      tooltip="{{T('COMPONENT.camera.cullingMask')}}"
    >
      <div id="checkboxes" class="layout vertical">
        <ui-checkbox :value="_everythingMask(index)" @change="_everythingMaskChanged($event, index)">
          Everything
        </ui-checkbox>
        <div v-for="(index, group) in groupList">
          <ui-checkbox :value="_cullingMask(index)" @change="_cullingMaskChanged($event, index)">
            {{group}}
          </ui-checkbox>
        </div>
      </div>
    </ui-prop>

    <ui-prop
      name="clearFlags"
      :multi-values="multi"
      auto-height
      tooltip="{{T('COMPONENT.camera.clearFlags')}}"
    >
      <div id="checkboxes" class="layout vertical">
        <div v-for="(index, name) in clearFlags">
          <ui-checkbox :value="_clearFlags(index)" @change="clearFlagsChanged($event, index)">
            {{name}}
          </ui-checkbox>
        </div>
      </div>
    </ui-prop>

    <ui-prop
      v-prop="target.rect"
      :multi-values="multi"
    ></ui-prop>

    <ui-prop
      v-prop="target.zoomRatio"
      :multi-values="multi"
    ></ui-prop>

    <ui-prop
      v-if = "is3DCamera()"
      name="renderStages"
      :multi-values="multi"
      auto-height
      tooltip="{{T('COMPONENT.camera.renderStages')}}"
    >
      <div id="checkboxes" class="layout vertical">
        <div v-for="(index, name) in renderStages">
          <ui-checkbox :value="_renderStages(index)" @change="renderStagesChanged($event, index)">
            {{name}}
          </ui-checkbox>
        </div>
      </div>
    </ui-prop>

    <ui-prop
      v-if = "is3DCamera()"
      v-prop="target.nearClip"
      :multi-values="multi"
    ></ui-prop>

    <ui-prop
      v-if = "is3DCamera()"
      v-prop="target.farClip"
      :multi-values="multi"
    ></ui-prop>

    <ui-prop
      v-prop="target.alignWithScreen"
      :multi-values="multi"
    ></ui-prop>

    <ui-prop
      v-if = "is3DCamera()"
      v-prop="target.ortho"
      :multi-values="multi"
    ></ui-prop>

    <ui-prop
      v-if="!isOrtho(target) && is3DCamera() && !alignWithScreen()"
      v-prop="target.fov"
      :multi-values="multi"
    ></ui-prop>

    <ui-prop
      v-if="isOrtho(target) && !alignWithScreen()"
      v-prop="target.orthoSize"
      :multi-values="multi"
    ></ui-prop>

    `,
    props: {
        target: {
            twoWay: !0,
            type: Object
        },
        multi: {
            type: Boolean
        }
    },
    data: ()=>({
        groupList: [],
        clearFlags: ["Color", "Depth", "Stencil"],
        renderStages: ["Opaque", "Transparent"]
    }),
    methods: {
        T: Editor.T,
        is3DCamera() {
            return this.target._is3D.value
        },
        alignWithScreen() {
            return this.target.alignWithScreen.value
        },
        isOrtho: e=>e.ortho.value,
        _setProperty(e, t) {
            var n = {
                id: this.target.uuid.value,
                path: t,
                type: "Number",
                isSubProp: !1,
                value: e
            };
            Editor.Ipc.sendToPanel("scene", "scene:set-property", n),
            Editor.Ipc.sendToPanel("scene", "scene:undo-commit")
        },
        _onProfileChanged() {
            this.groupList = this.profile.data["group-list"]
        },
        _cullingMask(e) {
            return !!(this.target.cullingMask.value & 1 << e)
        },
        _cullingMaskChanged(e, t) {
            let n = this.target.cullingMask.value
              , r = 0;
            for (let i = 0, a = 32; i < a; i++)
                r += i !== t ? n & 1 << i : e.currentTarget.value ? 1 << t : 0;
            this.target.cullingMask.value = r,
            this._setProperty(r, "cullingMask")
        },
        _everythingMask() {
            let e = this.target.cullingMask.value;
            for (let t = 0, n = this.groupList.length; t < n; t++)
                if (!(e & 1 << t))
                    return !1;
            return !0
        },
        _everythingMaskChanged(e) {
            let t = 0;
            e.currentTarget.value && (t = 4294967295),
            this.target.cullingMask.value = t,
            this._setProperty(t, "cullingMask")
        },
        _clearFlags(e) {
            return !!(this.target.clearFlags.value & 1 << e)
        },
        clearFlagsChanged(e, t) {
            let n = this.target.clearFlags.value
              , r = 0;
            for (let i = 0, a = this.clearFlags.length; i < a; i++)
                r += i !== t ? n & 1 << i : e.currentTarget.value ? 1 << t : 0;
            this.target.clearFlags.value = r,
            this._setProperty(r, "clearFlags")
        },
        _renderStages(e) {
            return !!(this.target.renderStages.value & 1 << e)
        },
        renderStagesChanged(e, t) {
            let n = this.target.renderStages.value
              , r = 0;
            for (let i = 0, a = this.renderStages.length; i < a; i++)
                r += i !== t ? n & 1 << i : e.currentTarget.value ? 1 << t : 0;
            this.target.renderStages.value = r,
            this._setProperty(r, "renderStages")
        }
    },
    compiled() {
        let e = Editor.Profile.load("project://project.json");
        this.profile = e,
        this.groupList = e.get("group-list"),
        e.on("changed", this._onProfileChanged)
    },
    destroyed() {
        this.profile && this.profile.removeListener("changed", this._onProfileChanged)
    }
});
