"use strict";
const e = Editor.require("packages://inspector/utils/utils").syncObjectProperty;
Vue.component("cc-gradient-range", {
    template: `
    '
    <div>
        <style type="text/css">
            @import url('packages://curve-editor/style/iconfont.css');
        </style>

        <div class="ui-gradient-range">
            <template
                v-if="target"
            >
                <ui-prop
                    :tooltip="target.attrs.tooltip"
                    :name="target.name"
                    :indent="indent"
                    v-readonly="target.attrs.readonly"
                >
                    <ui-select class="flex-1"
                        :value = "mode"
                        :multi-values = "multi"
                        @confirm="_onChangeMode"
                    >
                        <option value='0'>Color</option>
                        <option value='1'>Gradient</option>
                        <option value='2'>TwoColors</option>
                        <option value='3'>TwoGradients</option>
                        <option value='4'>RandomColor</option>
                    </ui-select>
                </ui-prop>

                <div class="prop-content">
                        <template
                            v-if="target.value && target.value.mode !== undefined"
                        >
                            <div class="content">
                                <template
                                    v-if="isOneColor()"
                                >
                                    <component auto="true"
                                        :is="target.value.color.compType"
                                        :target.sync="target.value.color"
                                        :indent="indent+1"
                                        :multi-values="multi"
                                    ></component>
                                </template>

                                <template
                                    v-if="isTwoColor()"
                                >
                                    <component auto="true"
                                        :is="target.value.colorMin.compType"
                                        :target.sync="target.value.colorMin"
                                        :indent="indent+1"
                                        :multi-values="multi"
                                    ></component>
                                    <component auto="true"
                                        :is="target.value.colorMax.compType"
                                        :target.sync="target.value.colorMax"
                                        :indent="indent+1"
                                        :multi-values="multi"
                                    ></component>
                                </template>

                                <template
                                    v-if="isTwoGradient()"
                                >
                                    <component auto="true"
                                        :is="target.value.gradientMin.compType"
                                        :target.sync="target.value.gradientMin"
                                        :indent="indent+1"
                                        :multi-values="multi"
                                    ></component>
                                    <component auto="true"
                                        :is="target.value.gradientMax.compType"
                                        :target.sync="target.value.gradientMax"
                                        :indent="indent+1"
                                        :multi-values="multi"
                                    ></component>
                                </template>

                                <template
                                    v-if="isOneGradient()"
                                >
                                    <component
                                        v-if="target.value.gradient.attrs.visible !== false"
                                        :is="target.value.gradient.compType"
                                        :target.sync="target.value.gradient"
                                        :indent="indent+1"
                                    ></component>
                                </template>
                            </div>
                        </template>

                </div>
            </template>
        </div>
    </div>'
    `,
    props: {
        title: {
            type: String
        },
        target: {
            type: Object
        },
        indent: {
            type: Number,
            default: 0
        },
        multi: {
            type: Boolean
        }
    },
    methods: {
        isOneColor() {
            return this.target.value.mode.value === cc.GradientRange.Mode.Color
        },
        isOneGradient() {
            return this.target.value.mode.value === cc.GradientRange.Mode.Gradient || this.target.value.mode.value === cc.GradientRange.Mode.RandomColor
        },
        isTwoColor() {
            return this.target.value.mode.value === cc.GradientRange.Mode.TwoColors
        },
        isTwoGradient() {
            return this.target.value.mode.value === cc.GradientRange.Mode.TwoGradients
        },
        _onChangeMode(t) {
            this.mode = parseInt(t.detail.value),
            e(this.$root, this.target.value.mode.path, "Enum", this.mode)
        }
    },
    data: ()=>({
        style: {
            background: "linear-gradient(to right, #fff 0%,#fff 100%)"
        },
        mode: 0
    }),
    created() {
        this.mode = this.target.value.mode.value
    },
    compiled() {}
});
