"use strict";
Vue.component("cc-node-header", {
    template:
    `
    <div :style="cssHost">
      <div :style="cssPrefab" class="layout horizontal center-center" v-if="target.__prefab__">
            <ui-button
              title="{{T('INSPECTOR.node.prefab_sync_mode')}}"
              class="tiny transparent green"
              v-if="target.__prefab__.sync"
              @confirm="prefabSetSync"
            >
              <i class="fa fa-link"></i>
            </ui-button>
            <ui-button
              title="{{T('INSPECTOR.node.prefab_unsync_mode')}}"
              class="tiny transparent blue"
              v-if="!target.__prefab__.sync"
              @confirm="prefabSetSync"
            >
              <i class="fa fa-unlink"></i>
            </ui-button>

            <div :style="cssLabel">Prefab: </div>
            <ui-button
              class="tiny transparent"
              :style="cssPrefabNameBtn"
              @confirm="prefabSelectRoot"
            >
              <span :style="cssPrefabName">{{target.__prefab__.rootName}}</span>
            </ui-button>

            <span class="flex-1"></span>

            <ui-button
              title="{{T('INSPECTOR.node.prefab_select')}}"
              class="tiny transparent"
              @confirm="prefabSelectAsset"
            >
              {{T('INSPECTOR.node.prefab_btn_select')}}
            </ui-button>
            <ui-button
              v-if="!target.__prefab__.subNode && !target.__prefab__.subPrefab"
              v-el:revert
              title="{{T('INSPECTOR.node.prefab_revert')}}"
              class="tiny transparent"
              @confirm="prefabRevert"
            >
              {{T('INSPECTOR.node.prefab_btn_revert')}}
            </ui-button>
            <ui-button
              v-if="!target.__prefab__.assetReadonly && !target.__prefab__.subNode && !target.__prefab__.subPrefab"
              v-el:save
              title="{{T('INSPECTOR.node.prefab_apply')}}"
              class="tiny transparent"
              @confirm="prefabApply"
            >
              {{T('INSPECTOR.node.prefab_btn_apply')}}
            </ui-button>
          </div>
          <div class="layout horizontal center-center">
            <ui-checkbox
              v-value="target.active.value"
              v-values="target.active.values"
              :multi-values="_updateActiveMulti(target.active)"
              tooltip="{{T('INSPECTOR.node.active')}}"
            ></ui-checkbox>
            <ui-input class="flex-1"
              v-value="target.name.value"
              v-values="target.name.values"
              :multi-values="_updateNameMulti(target.name)"
              tooltip="{{T('INSPECTOR.node.name')}}"
            >
            </ui-input>
            <ui-button class="tiny transparent"
              @click="toggle3D"
              :pressed="target.is3DNode.value"
              :style="target.is3DNode.value ? css3dBtnPressed : css3dBtn"
              title="{{'TODO'}}"
              >
              3D
            </ui-button>
          </div>
        </div>
`,
    data: ()=>({
        cssHost: {
            flex: "none",
            paddingBottom: "2px",
            margin: "5px 10px",
            marginBottom: "3px",
            overflow: "hidden"
        },
        cssPrefab: {
            borderBottom: "1px solid #666",
            marginBottom: "5px",
            paddingBottom: "1px"
        },
        cssLabel: {
            fontWeight: "bold",
            whiteSpace: "pre"
        },
        cssPrefabNameBtn: {
            flexShrink: 1,
            marginRight: "10px"
        },
        cssPrefabName: {
            overflow: "hidden",
            textOverflow: "ellipsis",
            whiteSpace: "nowrap"
        },
        css3dBtn: {
            "font-weight": "normal",
            "font-size": ".8rem",
            color: "rgb(170, 170, 170)",
            width: "29px"
        },
        css3dBtnPressed: {
            "font-weight": "normal",
            "font-size": ".8rem",
            color: "rgb(49, 157, 255)",
            background: "#454545",
            width: "29px",
            "box-shadow": "inset 0 2px 10px rgba(0, 0, 0, 0.5)",
            "transition-duration": "0.20s"
        }
    }),
    props: {
        target: {
            twoWay: !0,
            type: Object
        },
        multi: {
            type: Boolean
        }
    },
    methods: {
        T: Editor.T,
        toggle3D(e) {
            e.stopPropagation(),
            this.target.uuid.split("^").forEach(e=>{
                Editor.Ipc.sendToPanel("scene", "scene:set-property", {
                    id: e,
                    path: "is3DNode",
                    value: !this.target.is3DNode.value,
                    isSubProp: !1
                })
            }
            )
        },
        prefabSelectAsset() {
            Editor.UI.fire(this.$el, "prefab-select-asset")
        },
        prefabSelectRoot() {
            Editor.UI.fire(this.$el, "prefab-select-root")
        },
        prefabApply() {
            if (this.target.__prefab__.hasSubPrefab) {
                this._applyRequestID && (Editor.Ipc.cancelRequest(this._applyRequestID),
                this._applyRequestID = null);
                let e = this.$els.save.getBoundingClientRect();
                this._applyRequestID = Editor.Ipc.sendToPackage("inspector", "popup-multiple-prefab-menu", {
                    type: "apply",
                    uuid: this.target.uuid,
                    x: e.left,
                    y: e.bottom + 5,
                    nestedInfo: this.target.__prefab__.nestedInfo
                })
            } else
                Editor.UI.fire(this.$el, "prefab-apply")
        },
        prefabRevert() {
            if (this.target.__prefab__.hasSubPrefab) {
                this._revertRequestID && (Editor.Ipc.cancelRequest(this._revertRequestID),
                this._revertRequestID = null);
                let e = this.$els.save.getBoundingClientRect();
                this._revertRequestID = Editor.Ipc.sendToPackage("inspector", "popup-multiple-prefab-menu", {
                    type: "revert",
                    uuid: this.target.uuid,
                    x: e.left,
                    y: e.bottom + 5,
                    nestedInfo: this.target.__prefab__.nestedInfo
                })
            } else
                Editor.UI.fire(this.$el, "prefab-revert")
        },
        prefabSetSync() {
            Editor.UI.fire(this.$el, "prefab-set-sync")
        },
        _updateActiveMulti: e=>e.values.length > 1,
        _updateNameMulti: e=>e.values.length > 1
    }
});
