"use strict";
Vue.component("cc-node-section", {
    template:
    `
    <ui-section>
      <div class="header flex-1 layout horizontal center-center" >
        <span @dragstart="dragstart" @dragend="dragend" draggable="true">Node</span>
        <span class="flex-1"></span>
        <ui-button v-el:dropdown class="tiny transparent"
          title=""
          @click="menuClick">
          <i class="fa fa-cog"></i>
        </ui-button>
      </div>

      <ui-prop name="Position" :type="is3D() ? 'cc.Vec3' : 'cc.Vec2'" step="1"
        :multi-values="multi"
        v-value="target.position.value"
        v-values="target.position.values"
        v-readonly="target.position.readonly"
        tooltip="{{T('INSPECTOR.node.position')}}"
      ></ui-prop>

      <ui-prop name="Rotation" type="number" step="1" v-if="!is3D()"
        :multi-values="multi"
        v-value="target.angle.value"
        v-values="target.angle.values"
        v-readonly="target.angle.readonly"
        tooltip="{{T('INSPECTOR.node.rotation')}}"
      ></ui-prop>

      <ui-prop name="Rotation" type="cc.Vec3" v-if="is3D()"
        :multi-values="multi"
        v-value="target.eulerAngles.value"
        v-values="target.eulerAngles.values"
        v-readonly="target.eulerAngles.readonly"
        tooltip="{{T('INSPECTOR.node.rotation')}}"
      ></ui-prop>

      <ui-prop name="Scale" :type="is3D() ? 'cc.Vec3' : 'cc.Vec2'" step="0.1"
        :multi-values="multi"
        v-value="target.scale.value"
        v-values="target.scale.values"
        v-readonly="target.scale.readonly"
        tooltip="{{T('INSPECTOR.node.scale')}}"
      ></ui-prop>

      <ui-prop name="Anchor" type="cc.Vec2" step="0.1"
        :multi-values="multi"
        v-value="target.anchor.value"
        v-values="target.anchor.values"
        v-readonly="target.anchor.readonly"
        tooltip="{{T('INSPECTOR.node.anchor')}}"
      ></ui-prop>

      <ui-prop name="Size" type="cc.Size" step="1"
        :multi-values="multi"
        v-value="target.size.value"
        v-values="target.size.values"
        v-readonly="target.size.readonly"
        tooltip="{{T('INSPECTOR.node.size')}}"
      ></ui-prop>

      <ui-prop name="Color" type="cc.Color"
        :multi-values="multi"
        v-value="target.color.value"
        v-values="target.color.values"
        v-readonly="target.color.readonly"
        tooltip="{{T('INSPECTOR.node.color')}}"
      ></ui-prop>

      <ui-prop name="Opacity" type="number" min="0" max="255" step="1"
        :multi-values="multi"
        v-value="target.opacity.value"
        v-values="target.opacity.values"
        v-readonly="target.opacity.readonly"
        tooltip="{{T('INSPECTOR.node.opacity')}}"
      ></ui-prop>

      <ui-prop name="Skew" type="cc.Vec2" step="1"
        :multi-values="multi"
        v-value="target.skew.value"
        v-values="target.skew.values"
        v-readonly="target.skew.readonly"
      ></ui-prop>

      <ui-prop name="Group" tooltip="{{T('INSPECTOR.node.group')}}">
        <ui-select class="flex-1"
          :multi-values="_updateGroupMultiValues(target.group)"
          v-value="target.group.value"
          v-values="target.group.values"
          v-readonly="target.group.readonly"
          v-on:change="_groupChanged(target.group.value)"
        >
          <option v-for="g in groupList" value="{{g}}">{{g}}</option>
        </ui-select>
        <ui-button class="tiny blue" @confirm="openGroupSettings">
          {{T('COMPONENT.sprite.edit_button')}}
        </ui-button>
      </ui-prop>
    </ui-section>

    <cc-comp-section
      v-for="comp in target.__comps__"
      :name.sync="comp.__displayName__"
      :editor.sync="comp.__editor__"
      :target.sync="comp.value"
      :multi="multi"
      :index="$index"
      :count="target.__comps__.length"
    >
    </cc-comp-section>
`,
    props: {
        target: {
            twoWay: !0,
            type: Object
        },
        multi: {
            twoWay: !1,
            type: Boolean
        }
    },
    data: ()=>({
        groupList: []
    }),
    methods: {
        T: Editor.T,
        _groupChanged(e) {
            Editor.Ipc.sendToPanel("scene", "scene:set-group-sync", this.target.uuid, e)
        },
        openGroupSettings() {
            Editor.Ipc.sendToMain("project-settings:open", {
                tab: 0
            })
        },
        menuClick(e) {
            e.stopPropagation(),
            this._requestID && (Editor.Ipc.cancelRequest(this._requestID),
            this._requestID = null),
            this._requestID = Editor.Ipc.sendToPanel("scene", "scene:has-copied-component", (e,t)=>{
                let n = this.$els.dropdown.getBoundingClientRect();
                Editor.Ipc.sendToPackage("inspector", "popup-node-inspector-menu", {
                    uuids: this.target.uuids,
                    hasCopyComp: t,
                    x: n.left,
                    y: n.bottom + 5
                })
            }
            , -1)
        },
        dragstart(e) {
            e.stopPropagation(),
            Editor.UI.DragDrop.start(e.dataTransfer, {
                buildImage: !0,
                effect: "copyMove",
                type: "node",
                items: [{
                    id: this.target.uuid,
                    name: this.target.name.value
                }]
            })
        },
        dragend() {
            Editor.UI.DragDrop.end()
        },
        _updateGroupMultiValues(e) {
            if (!e || e.values.length <= 1)
                return !1;
            var t = e.values[0];
            return !e.values.every(e=>e == t)
        },
        _onProfileChanged() {
            this.groupList = this.profile.get("group-list")
        },
        is3D() {
            return "cc.Vec3" === this.target.position.type
        }
    },
    compiled() {
        let e = Editor.Profile.load("project://project.json");
        this.profile = e,
        this.groupList = e.get("group-list"),
        e.on("changed", this._onProfileChanged)
    },
    destroyed() {
        this.profile && this.profile.removeListener("changed", this._onProfileChanged)
    }
});
