"use strict";
Vue.component("cc-sprite-section", {
    template:
    `

    <style>
    .line {
        width: 100%;
        margin-top: 5px;
        margin-bottom: 5px;
    }
    </style>

    <ui-prop name="Trim Type" type="enum"
      v-value="target.trimType"
      v-values="multiTrimType"
      :multi-values="multi"
      :path="getPath('trimType')"
    >
      <option value="auto">Auto</option>
      <option value="custom">Custom</option>
      <option value="none">None</option>
    </ui-prop>

    <ui-prop name="Trim Threshold" type="number"
      v-value="target.trimThreshold"
      v-values="multiTrimThreshold"
      :multi-values="multi"
      :path="getPath('trimThreshold')"
      v-disabled="isNone()"
    ></ui-prop>
    <ui-prop name="Trim X" type="number"
      v-value="target.trimX"
      v-values="multiTrimX"
      :multi-values="multi"
      :path="getPath('trimX')"
      v-disabled="!isCustom()||isNone()"
    ></ui-prop>
    <ui-prop name="Trim Y" type="number"
      v-value="target.trimY"
      v-values="multiTrimY"
      :multi-values="multi"
      :path="getPath('trimY')"
      v-disabled="!isCustom()||isNone()"
    ></ui-prop>
    <ui-prop name="Trim Width" type="number"
      v-value="target.width"
      v-values="multiWidth"
      :multi-values="multi"
      :path="getPath('width')"
      v-disabled="!isCustom()||isNone()"
    ></ui-prop>
    <ui-prop name="Trim Height" type="number"
      v-value="target.height"
      v-values="multiHeight"
      :multi-values="multi"
      :path="getPath('height')"
      v-disabled="!isCustom()||isNone()"
    ></ui-prop>

    <div class="line"></div>

    <ui-prop name="Border Top" type="number"
      v-value="target.borderTop"
      v-values="multiComputeBoraderTop"
      :multi-values="multi"
      :path="getPath('borderTop')"
      v-on:change="onComputeBorderTop"
    ></ui-prop>
    <ui-prop name="Border Bottom" type="number"
      v-value="target.borderBottom"
      v-values="multiComputeBoraderBottom"
      :multi-values="multi"
      :path="getPath('borderBottom')"
      v-on:change="onComputeBorderBottom"
    ></ui-prop>
    <ui-prop name="Border Left" type="number"
      v-value="target.borderLeft"
      v-values="multiComputeBoraderLeft"
      :multi-values="multi"
      :path="getPath('borderLeft')"
      v-on:change="onComputeBorderLeft"
    ></ui-prop>
    <ui-prop name="Border Right" type="number"
      v-value="target.borderRight"
      v-values="multiComputeBoraderRight"
      :multi-values="multi"
      :path="getPath('borderRight')"
      v-on:change="onComputeBorderRight"
    ></ui-prop>

    <div class="line"></div>

    <ui-prop name="Rotated" type="boolean"
      v-value="target.rotated"
      v-values="multiRotated"
      :multi-values="multi"
      v-disabled="true"
    ></ui-prop>
    <ui-prop name="Offset X" type="number"
      v-value="target.offsetX"
      v-values="multiOffsetX"
      :multi-values="multi"
      v-disabled="true"
    ></ui-prop>
    <ui-prop name="Offset Y" type="number"
      v-value="target.offsetY"
      v-values="multiOffsetY"
      :multi-values="multi"
      v-disabled="true"
    ></ui-prop>

    <ui-button v-if="!multi" :style="cssBtn" class="blue" @confirm="editSprite">{{ T('COMPONENT.sprite.edit_button') }}</ui-button>

    `,
    data: () => ({
        cssBtn: { display: "block", width: "240px", margin: "10px auto" },
    }),
    props: {
        target: { twoWay: !0, type: Object },
        multi: Boolean,
        child: Boolean,
    },
    computed: {
        multiRotated: {
            get() {
                return this.target.multiValues
                    ? this.target.multiValues.rotated
                    : [];
            },
        },
        multiOffsetX: {
            get() {
                return this.multi ? this.target.multiValues.offsetX : [];
            },
        },
        multiOffsetY: {
            get() {
                return this.multi ? this.target.multiValues.offsetY : [];
            },
        },
        multiTrimType: {
            get() {
                return this.multi ? this.target.multiValues.trimType : [];
            },
        },
        multiTrimThreshold: {
            get() {
                return this.multi ? this.target.multiValues.trimThreshold : [];
            },
        },
        multiTrimX: {
            get() {
                return this.multi ? this.target.multiValues.trimX : [];
            },
        },
        multiTrimY: {
            get() {
                return this.multi ? this.target.multiValues.trimY : [];
            },
        },
        multiWidth: {
            get() {
                return this.multi ? this.target.multiValues.width : [];
            },
        },
        multiHeight: {
            get() {
                return this.multi ? this.target.multiValues.height : [];
            },
        },
        multiComputeBoraderLeft: {
            get() {
                return this.multi ? this.target.multiValues.borderLeft : [];
            },
        },
        multiComputeBoraderRight: {
            get() {
                return this.multi ? this.target.multiValues.borderRight : [];
            },
        },
        multiComputeBoraderBottom: {
            get() {
                return this.multi ? this.target.multiValues.borderRight : [];
            },
        },
        multiComputeBoraderTop: {
            get() {
                return this.multi ? this.target.multiValues.borderTop : [];
            },
        },
    },
    methods: {
        T: Editor.T,
        getPath(t) {
            return this.child ? `target.subMetas.0.${t}` : "";
        },
        deltaWidth: (t) => Math.ceil((t.rawWidth - t.width) / 2),
        deltaHeight: (t) => Math.ceil((t.rawHeight - t.height) / 2),
        onComputeBorderTop(t) {
            this.target.borderTop = Math.max(
                t.detail.value -
                    this.deltaHeight(this.target) -
                    this.target.offsetY,
                0
            );
        },
        onComputeBorderBottom(t) {
            this.target.borderBottom = Math.max(
                t.detail.value -
                    this.deltaHeight(this.target) -
                    this.target.offsetY,
                0
            );
        },
        onComputeBorderLeft(t) {
            this.target.borderLeft = Math.max(
                t.detail.value -
                    this.deltaWidth(this.target) -
                    this.target.offsetX,
                0
            );
        },
        onComputeBorderRight(t) {
            this.target.borderRight = Math.max(
                t.detail.value -
                    this.deltaWidth(this.target) -
                    this.target.offsetX,
                0
            );
        },
        editSprite() {
            Editor.Panel.open("sprite-editor", { uuid: this.target.uuid });
        },
        isNone() {
            return "none" === this.target.trimType;
        },
        isCustom() {
            return "custom" === this.target.trimType;
        },
    },
});
