"use strict";
Vue.component("texture-format", {
    template: `

    <style>
        .wrapper {
                display: flex;
                flex-direction: column;
                padding: 5px 15px;
                border-top: rgba(0, 0, 0, 0.58) dashed 1px;
        }
    </style>
    <div style="min-height: 40px;">
        <div
            style="margin-top: 10px; margin-bottom: 10px"
            class="layout horizontal center-center"
        >
            <ui-select
                style="width: 240px;"
                @confirm="onSelectFormat"
                v-value="selectValue"
            >
                <option value="none">Select A Format To Add</option>
                <option
                    v-for="(name, description) in platformSupportedFormats"
                    :value="name"
                >
                    {{description}}
                </option>
            </ui-select>
        </div>
        <div>
            <div
                v-for="(index, format) in formats"
                class="wrapper"
            >
                <div style="display:flex;">
                    <span style="flex:1;">{{supportFormats[format.name]}}</span>
                    <ui-button class="mini" @confirm="deleteFormat($event, format)">
                        x
                    </ui-button>
                </div>

                \x3c!-- pvr format quality --\x3e
                <ui-prop
                    type="enum"
                    name="Quality"
                    v-value="format.quality"
                    :path="getPath(index, 'quality')"
                    v-if="isPVRFormat(format.name)"
                >
                    <option value="fastest">Fastest</option>
                    <option value="fast">Fast</option>
                    <option value="normal">Normal</option>
                    <option value="high">High</option>
                    <option value="best">Best</option>
                </ui-prop>

                \x3c!-- etc format quality --\x3e
                <ui-prop
                    type="enum"
                    name="Quality"
                    v-value="format.quality"
                    :path="getPath(index, 'quality')"
                    v-if="isETCFormat(format.name)"
                >
                    <option value="slow">Slow</option>
                    <option value="fast">Fast</option>
                </ui-prop>

                \x3c!-- normal format quality --\x3e
                <ui-prop
                    type="number"
                    name="Quality"
                    v-value="format.quality"
                    :path="getPath(index, 'quality')"
                    min="1" max="99"
                    v-if="!isPVRFormat(format.name) && !isETCFormat(format.name)"
                >
                </ui-prop>

                <div v-if="isETC2Format(format.name) && platform === 'android'" style="margin-top:10px;color:yellow;">
                    <i class="fa fa-exclamation-triangle fa-1" aria-hidden="true" style="margin-right:10px;"></i>
                    {{T('INSPECTOR.texture_compress.etc2_warning')}}
                </div>
            </div>
        </div>
    </div>

    `,
    props: {
        target: {
            twoWay: !0,
            type: Object
        },
        settings: {
            twoWay: !0,
            type: Object
        },
        platform: {
            type: String
        }
    },
    data: function() {
        return {
            selectValue: "none",
            formats: [],
            supportFormats: {
                png: "PNG",
                jpg: "JPG",
                webp: "Webp",
                pvrtc_4bits: "PVRTC 4bits RGBA",
                pvrtc_4bits_rgb: "PVRTC 4bits RGB",
                pvrtc_4bits_rgb_a: "PVRTC 4bits RGB Separate A",
                pvrtc_2bits: "PVRTC 2bits RGBA",
                pvrtc_2bits_rgb: "PVRTC 2bits RGB",
                pvrtc_2bits_rgb_a: "PVRTC 2bits RGB Separate A",
                etc1: "Etc1 RGB Separate A",
                etc1_rgb: "Etc1 RGB",
                etc2: "Etc2 RGBA",
                etc2_rgb: "Etc2 RGB"
            }
        }
    },
    watch: {
        settings() {
            this.formats = this.settings[this.platform] ? this.settings[this.platform].formats : []
        }
    },
    compiled() {
        this.formats = this.settings[this.platform] ? this.settings[this.platform].formats : []
    },
    computed: {
        platformSupportedFormats() {
            let t = {};
            for (let n in this.supportFormats)
                this.supportFormat(n) && (t[n] = this.supportFormats[n]);
            return t
        }
    },
    methods: {
        getPath(t, n) {
            return `target.platformSettings.${this.platform}.formats.${t}.${n}`
        },
        onSelectFormat(t) {
            if ("none" === this.selectValue)
                return;
            let n = this.settings[this.platform];
            n || (n = this.settings[this.platform] = {
                formats: []
            },
            this.formats = n.formats);
            let a = n.formats.find(t=>this.selectValue.startsWith("etc") ? t.name.startsWith("etc") : this.selectValue.startsWith("pvr") ? t.name.startsWith("pvr") : t.name === this.selectValue);
            if (a)
                return Editor.warn("Already adds a same format."),
                void 0;
            a = {
                name: this.selectValue,
                quality: this.defaultQuality(this.selectValue)
            },
            this.formats.push(a),
            Editor.UI._DomUtils.fire(t.currentTarget, "change", {
                bubbles: !1,
                detail: {
                    path: "target.platformSettings",
                    value: this.settings
                }
            }),
            setTimeout(()=>{
                this.selectValue = "none"
            }
            , 0)
        },
        _openExternal(t, n) {
            t.stopPropagation(),
            require("electron").shell.openExternal(n)
        },
        T: t=>Editor.T(t),
        isPVRFormat: t=>t.startsWith("pvrtc_"),
        isETC2Format: t=>t.startsWith("etc2"),
        isETCFormat: t=>t.startsWith("etc1") || t.startsWith("etc2"),
        defaultQuality(t) {
            return this.isPVRFormat(t) ? "normal" : this.isETCFormat(t) ? "fast" : 80
        },
        deleteFormat(t, n) {
            let a = this.settings[this.platform].formats.findIndex(t=>t.name === n.name);
            -1 !== a && (this.formats.splice(a, 1),
            Editor.UI._DomUtils.fire(t.currentTarget, "change", {
                bubbles: !1,
                detail: {
                    path: "target.platformSettings",
                    value: this.settings
                }
            }))
        },
        supportFormat(t) {
            return ("pvrtc" !== (t = t.split("_")[0]) || "android" !== this.platform && "default" !== this.platform) && (("etc1" !== t || "ios" !== this.platform && "default" !== this.platform) && ("etc2" !== t || "android" === this.platform || "ios" === this.platform))
        }
    }
}),
Vue.component("texture-compress", {
    template: '\n    <style>\n        .compress-option {\n            margin-top: 15px;\n            margin-bottom: 10px;\n            margin-left: 10px;\n            background-color: rgba(105, 105, 105, 0.45);\n        }\n\n        .compress-option ui-prop {\n            margin-right: 10px;\n        }\n\n        .compress-tab {\n            display: flex;\n            flex: 1;\n            flex-direction: row;\n            border-bottom: rgba(43, 40, 40, 0.52) solid 2px;\n        }\n\n        .compress-tab span {\n            flex: 1;\n            text-align: center;\n            padding: 10px 0px;\n        }\n\n        .compress-tab span.icon {\n            font-size: 15px;\n        }\n\n        .compress-tab span.active {\n            background-color: black;\n        }\n    </style>\n\n    <div class="compress-option">\n        <div class="compress-tab flex layout row flex-1">\n            <span \n                @click="changePlatform(\'default\')"    \n                :class="activeClass(\'default\')"                         \n            >\n                Default\n            </span>\n            <span \n                @click="changePlatform(\'android\')"    \n                :class="activeClass(\'android\')"\n                title="Android"\n            >\n                Android\n            </span>\n            <span \n                @click="changePlatform(\'ios\')"        \n                :class="activeClass(\'ios\')"     \n                title="iOS"\n            >\n                iOS\n            </span>\n            <span \n                @click="changePlatform(\'web\')"        \n                :class="activeClass(\'web\')"\n                title="Web"\n            >\n                Web\n            </span>\n            <span \n                @click="changePlatform(\'minigame\')"\n                :class="activeClass(\'minigame\')"\n                title="Mini Game"\n            >\n                Mini Game\n            </span>\n        </div>\n        <div v-for="platform in platforms" :class="\'compress-\' + platform">\n            <texture-format :target.sync="target" :platform="platform" :settings.sync="target.platformSettings" v-if="currentPlatform === platform"/>\n        </div>\n    </div>\n    ',
    props: {
        target: {
            twoWay: !0,
            type: Object
        }
    },
    data: function() {
        return {
            currentPlatform: this.$parent.$options.profiles.project.get("current-platform"),
            platforms: ["default", "android", "ios", "minigame", "web"]
        }
    },
    methods: {
        changePlatform(t) {
            this.currentPlatform = t,
            this.$parent.$options.profiles.project.set("current-platform", t)
        },
        activeClass(t) {
            return this.currentPlatform === t ? "active" : ""
        },
        isPVRTC: t=>-1 !== t.indexOf("pvrtc_")
    }
});
